@@ -45,9 +45,9 @@ documentation and/or software.
 #define S44 21
 
 static void MD5Transform PROTO_LIST ((UINT4 [4], unsigned char [64]));
-static void Encode PROTO_LIST
+void Encode PROTO_LIST
   ((unsigned char *, UINT4 *, unsigned int));
-static void Decode PROTO_LIST
+void Decode PROTO_LIST
   ((UINT4 *, unsigned char *, unsigned int));
 
 static unsigned char PADDING[64] = {
@@ -91,30 +91,32 @@ Rotation is separate from addition to prevent recomputation.
  (a) += (b); \
   }
 
-/* MD5 initialization. Begins an MD5 operation, writing a new context.
- */
-void MD5Init (context)
-MD5_CTX *context;                                        /* context */
-{
+void MD5Init(context)
+MD5_CTX *context;
+ {
+  /* MD5 initialisation, writing a new context */
   context->count[0] = context->count[1] = 0;
-  /* Load magic initialization constants.
-*/
+  /* Load magic initialization constants.*/
   context->state[0] = 0x67452301;
   context->state[1] = 0xefcdab89;
   context->state[2] = 0x98badcfe;
   context->state[3] = 0x10325476;
-}
+ }
+
+
+
 
 /* MD5 block update operation. Continues an MD5 message-digest
   operation, processing another message block, and updating the
-  context.
- */
+  context.*/
+
 void MD5Update (context, input, inputLen)
 MD5_CTX *context;                                        /* context */
 unsigned char *input;                                /* input block */
 unsigned int inputLen;                     /* length of input block */
 {
-  unsigned int i, index, partLen;
+  register unsigned int i, index, partLen;
+
 
   /* Compute number of bytes mod 64 */
   index = (unsigned int)((context->count[0] >> 3) & 0x3F);
@@ -122,88 +124,87 @@ unsigned int inputLen;                     /* length of input block */
   /* Update number of bits */
   if ((context->count[0] += ((UINT4)inputLen << 3))
    < ((UINT4)inputLen << 3))
- context->count[1]++;
+  context->count[1]++;
   context->count[1] += ((UINT4)inputLen >> 29);
 
   partLen = 64 - index;
 
-  /* Transform as many times as possible.
-*/
-  if (inputLen >= partLen) {
- memcpy
-   ((POINTER)&context->buffer[index], (POINTER)input, partLen);
- MD5Transform (context->state, context->buffer);
-
- for (i = partLen; i + 63 < inputLen; i += 64)
-   MD5Transform (context->state, &input[i]);
-
- index = 0;
+  /* Transform as many times as possible.*/
+  if (inputLen >= partLen) 
+  {
+   memcpy
+       ((POINTER)&context->buffer[index], (POINTER)input, partLen);
+   MD5Transform (context->state, context->buffer);
+   for (i = partLen; i + 63 < inputLen; i += 64)
+      MD5Transform (context->state, &input[i]);
+   index = 0;
   }
   else
- i = 0;
-
   /* Buffer remaining input */
   memcpy
- ((POINTER)&context->buffer[index], (POINTER)&input[i],
-  inputLen-i);
+ ((POINTER)&context->buffer[index], (POINTER)&input[0],
+  inputLen);
 }
 
+
+
+
+
 /* MD5 finalization. Ends an MD5 message-digest operation, writing the
-  the message digest and zeroizing the context.
- */
-void MD5Final (digest, context)
-unsigned char digest[16];                         /* message digest */
+  the message digest and zeroizing the context.*/
+
+unsigned int *MD5Final (context)
 MD5_CTX *context;                                       /* context */
 {
   unsigned char bits[8];
-  unsigned int index, padLen;
+  register unsigned int index, padLen, i, j;
 
   /* Save number of bits */
-  Encode (bits, context->count, 8);
+   Encode (bits, context->count, 8);
+
+  /* Pad out to 56 mod 64. */
 
-  /* Pad out to 56 mod 64.
-*/
   index = (unsigned int)((context->count[0] >> 3) & 0x3f);
   padLen = (index < 56) ? (56 - index) : (120 - index);
   MD5Update (context, PADDING, padLen);
 
   /* Append length (before padding) */
   MD5Update (context, bits, 8);
-  /* Store state in digest */
-  Encode (digest, context->state, 16);
-
-  /* Zeroize sensitive information.
-*/
-  memset ((POINTER)context, 0, sizeof (*context));
+  
+  return((unsigned int*)context->state);
 }
 
+
+
+
 /* MD5 basic transformation. Transforms state based on block.
  */
 static void MD5Transform (state, block)
 UINT4 state[4];
 unsigned char block[64];
 {
-  UINT4 a = state[0], b = state[1], c = state[2], d = state[3], x[16];
+  register UINT4 a = state[0], b = state[1], c = state[2], d = state[3];
+  UINT4  x[16];
 
   Decode (x, block, 64);
 
   /* Round 1 */
-  FF (a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */
-  FF (d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */
-  FF (c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
-  FF (b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
-  FF (a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
-  FF (d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
-  FF (c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
-  FF (b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
-  FF (a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
-  FF (d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */
-  FF (c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
-  FF (b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
-  FF (a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
-  FF (d, a, b, c, x[13], S12, 0xfd987193); /* 14 */
-  FF (c, d, a, b, x[14], S13, 0xa679438e); /* 15 */
-  FF (b, c, d, a, x[15], S14, 0x49b40821); /* 16 */
+  FF (a, b, c, d, *x, S11, 0xd76aa478); /* 1 */
+  FF (d, a, b, c, *(x+1), S12, 0xe8c7b756); /* 2 */
+  FF (c, d, a, b, *(x+2), S13, 0x242070db); /* 3 */
+  FF (b, c, d, a, *(x+3), S14, 0xc1bdceee); /* 4 */
+  FF (a, b, c, d, *(x+4), S11, 0xf57c0faf); /* 5 */
+  FF (d, a, b, c, *(x+5), S12, 0x4787c62a); /* 6 */
+  FF (c, d, a, b, *(x+6), S13, 0xa8304613); /* 7 */
+  FF (b, c, d, a, *(x+7), S14, 0xfd469501); /* 8 */
+  FF (a, b, c, d, *(x+8), S11, 0x698098d8); /* 9 */
+  FF (d, a, b, c, *(x+9), S12, 0x8b44f7af); /* 10 */
+  FF (c, d, a, b, *(x+10), S13, 0xffff5bb1); /* 11 */
+  FF (b, c, d, a, *(x+11), S14, 0x895cd7be); /* 12 */
+  FF (a, b, c, d, *(x+12), S11, 0x6b901122); /* 13 */
+  FF (d, a, b, c, *(x+13), S12, 0xfd987193); /* 14 */
+  FF (c, d, a, b, *(x+14), S13, 0xa679438e); /* 15 */
+  FF (b, c, d, a, *(x+15), S14, 0x49b40821); /* 16 */
 
  /* Round 2 */
   GG (a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
@@ -263,16 +264,12 @@ unsigned char block[64];
   state[1] += b;
   state[2] += c;
   state[3] += d;
-
-  /* Zeroize sensitive information.
-*/
-  memset ((POINTER)x, 0, sizeof (x));
 }
 
 /* Encodes input (UINT4) into output (unsigned char). Assumes len is
-  a multiple of 4.
- */
-static void Encode (output, input, len)
+  a multiple of 4. */
+
+void Encode (output, input, len)
 unsigned char *output;
 UINT4 *input;
 unsigned int len;
@@ -288,9 +285,9 @@ unsigned int len;
 }
 
 /* Decodes input (unsigned char) into output (UINT4). Assumes len is
-  a multiple of 4.
- */
-static void Decode (output, input, len)
+  a multiple of 4.*/
+
+void Decode (output, input, len)
 UINT4 *output;
 unsigned char *input;
 unsigned int len;
