@@ -17,12 +17,15 @@
 
    Version 0.1   Date: 21 Feb 2001.
    Version 0.2   Date: 03 Mar 2001.
+   Version 0.3   Date: 05 Mar 2001.
+   Version 0.4   Date: 07 Mar 2001.
 */
 
 
 #include <stdio.h>
 #include <unistd.h>
 #include <time.h>
+#include <sys/timeb.h>
 #include <string.h>
 #include <sys/types.h>
 #include <signal.h>
@@ -30,27 +33,47 @@
 
 
 
-void hashprint(unsigned char[]);
+void hashprint(unsigned int *);
 
 unsigned int count;
 unsigned int verbosity=0;
 unsigned int custom=0;
 unsigned int all=0;
 int counter=0;
-int elapsed;
+time_t elapsed;
+short elapsed_u;
 char target[33];
-char *crack();
+int crack();
+int crack_verbose();
 char *alfa;
 char test[9];
 extern int optind;
 extern char *optarg;
+extern void Decode(unsigned int *, unsigned char *, int);
+extern void Encode(unsigned char *, unsigned int *, int);
 time_t start;
+unsigned short start_u;
+
+
 
 void handle()
 {
  FILE *des;
- elapsed=(unsigned int)(time(NULL)-start);
- if (!elapsed) elapsed=1;
+ struct timeb *ft;
+ unsigned long diff;
+
+ ft=(struct timeb *) malloc(sizeof(struct timeb));
+ ftime(ft);
+ elapsed=(ft->time)-start;
+ elapsed_u=(ft->millitm)-start_u;
+ if (elapsed_u < 0)
+	{
+	 elapsed_u=1000+elapsed_u;
+         if (elapsed>0) elapsed--;
+	}
+
+ diff=(elapsed*1000)+elapsed_u;
+
  des=fopen("/tmp/.mdcrack.resume", "w+");
  if (des)
   {
@@ -58,7 +81,7 @@ void handle()
    fclose(des);
   }
  printf("\n\nGenerating resume file /tmp/.mdcrack.resume");
- printf("\n###########################################\n%s <-- string \nCollision(s) tested : %u in %u second(s).\nAverage of %u tests/sec.\n\n", test, count, elapsed, count/elapsed);
+ printf("\n###########################################\n%s <-- string \nCollision(s) tested : %u in %u second(s), %u millisec.\nAverage of %u tests/sec.\n\n", test, count, elapsed, elapsed_u, (unsigned int) ((double) (count/diff))*1000);
 
  fflush(stdout);
  exit (0);
@@ -69,7 +92,7 @@ void handle()
 
 void usage(void)
 {
- printf("\nMDCrack version 0.2\nT00l to bruteforce password MD5 hashes.\n\nUsage: MDCrack [-h] [-v] [-a] [-s string] [digest] \n-h : (h)elp, this text\n-v : set (v)erbose mode\n-a : find (a)ll collisions (won't stop to the first found)\n-s string : feed a custom characters (s)tring\nNo option at all will force mdcrack to resume a previously stopped session\n\nAuthor: Gregory Duchemin ( c3rb3r@hotmail.com ).\n\n*******************************************\nderived from the RSA Data Security Inc.\nMD5 Message-Digest Algorithm\n*******************************************\n\n");
+ printf("\nMDCrack version 0.4\nT00l to bruteforce password MD5 hashes.\n\nUsage: MDCrack [-h] [-v] [-a] [-s string] [digest] \n-h : (h)elp, this text\n-v : set (v)erbose mode\n-a : find (a)ll collisions (won't stop to the first found)\n-s string : feed a custom characters (s)tring\nNo option at all will force mdcrack to resume a previously stopped session\n\nAuthor: Gregory Duchemin ( c3rb3r@hotmail.com ).\n\n*******************************************\nderived from the RSA Data Security Inc.\nMD5 Message-Digest Algorithm\n*******************************************\n\n");
 }
 
 
@@ -79,7 +102,7 @@ int main (argc, argv)
      char *argv[];
 {
   int i;
-  char *found;
+  int found;
   FILE *des1;
   char c;
   char resume[9];
@@ -138,7 +161,7 @@ int main (argc, argv)
 
       case 's':
       custom=1;
-      realloc(alfa, strlen(optarg)+1);
+      realloc(alfa, strlen(optarg)+2);
       strcpy(alfa+1, optarg); 
       *(alfa)='\0';
       break;
@@ -157,20 +180,32 @@ int main (argc, argv)
 
 if (optind+1<=argc)
  {
-  if (strlen(*(argv+optind)) != 32)
+   if (strlen(*(argv+optind)) != 32)
     {
-      printf("\n\n not a md5 digest....must have 16 bytes length (32 ascii digits from 0 to F).\n\n");
+      printf("\n\nnot a md5 digest....must have 16 bytes length (32 ascii digits from 0 to F).\n\n");
       return(1);
     }
-  if ((verbosity) && (custom)) 
-      printf("\n\n Using custom string : %s \n\n", alfa+1);
-    if (!(found=crack(*(argv+optind), resume)))
+   if (custom) 
+    { 
+     printf("\n\nUsing custom string : %s \n\n", alfa+1);
+     sleep(2);
+    }
+   if (verbosity)
     {
+     if (!(found=crack_verbose(*(argv+optind), resume)))
+      {
       printf("\nsorry, did not succeed to find at least one collision.\n\n");
       return(1);
-    }  
-  return (0);
- }
+      }  
+     return (0);
+    }
+   if (!(found=crack(*(argv+optind), resume)))
+   {
+    printf("\nsorry, did not succeed to find at least one collision.\n\n");
+    return(1);
+   }  
+    return (0);
+  }
  else
  {
   usage();
@@ -213,22 +248,27 @@ int up2low(input)
 }
 
 
-char *crack(input, resume)
+int crack(input, resume)
      char *input;
      char *resume;
 {
   char conv[]="0123456789abcdef";
   register char *a,*b,*c,*d,*e,*f,*g,*h;
-  register char *result;
+  unsigned int *result2;
   char *copy;
   register int MAX=(unsigned int) alfa+strlen(alfa+1)+1;
   register int MIN=(unsigned int) alfa+1; 
-  unsigned char i,j,k; 
+  unsigned char i,j; 
   char *mov;
   MD5_CTX context;
   char digest[16];
-
+  unsigned int digest2[4];
+  struct timeb *tp;
+  struct timeb *ft;
+  unsigned long diff;
  
+  ft=(struct timeb *)malloc(sizeof(struct timeb));
+  tp=(struct timeb *)malloc(sizeof(struct timeb)); 
   bzero(digest, 16);
   mov=digest;
   copy=input;
@@ -257,14 +297,14 @@ char *crack(input, resume)
 
   up2low(input);
   ascii2bin(copy, conv, mov);
-    
+  Decode(digest2, mov, 16);  
   
-  result=(char *)malloc(20);
 
   bzero(test, 9);
   signal(SIGINT, handle);
-  MD5Init (&context);
-  start=time(NULL);
+  ftime(tp);
+  start=tp->time;
+  start_u=tp->millitm;
  
 
   
@@ -293,31 +333,33 @@ char *crack(input, resume)
 			      for(;h<(char *)MAX;h++)
 				{
 				  *test=*h;
-                                  MD5Init (&context);
+				  *(context.count)=*(context.count+1)=0;
+				  *(context.state)=0x67452301;
+				  *(context.state+1)=0xefcdab89;
+				  *(context.state+2)=0x98badcfe;
+				  *(context.state+3)=0x10325476;
                                   MD5Update (&context, test, strlen(test));
-                                  MD5Final (result, &context);
-                                  if (verbosity)
-                                    {
-				      puts("\n");
-                                      printf("%s ",test);
-                                      puts(" <-- string.");
-                                      hashprint (result);
-                                      puts(" <-- hash.");  
-                                      hashprint(digest);
-                                      puts(" <-- reference.\n\n");
-                                      fflush(stdout);
-				      }
+                                  result2=MD5Final(&context);
+
                                   count++;
-				  if ((*digest==*result)&&(*(digest+1)==*(result+1))&&(*(digest+2)==*(result+2))&&(*(digest+3)==*(result+3))&&(*(digest+4)==*(result+4))&&(*(digest+5)==*(result+5))&&(*(digest+6)==*(result+6))&&(*(digest+7)==*(result+7))&&(*(digest+8)==*(result+8))&&(*(digest+9)==*(result+9))&&(*(digest+10)==*(result+10))&&(*(digest+11)==*(result+11))&&(*(digest+12)==*(result+12))&&(*(digest+13)==*(result+13))&&(*(digest+14)==*(result+14))&&(*(digest+15)==*(result+15)))
+				  if ((*digest2==*result2)&&(*(digest2+1)==*(result2+1))&&(*(digest2+2)==*(result2+2))&&(*(digest2+3)==*(result2+3)))
 				    {
-				      printf("\n**********************************************\nCollision found ! => %s\n", test);  
-                                      elapsed=(unsigned int)(time(NULL)-start);
-                                      if (!elapsed) elapsed=1;
-                                      printf("\n\nCollision(s) tested : %u in %u second(s).\nAverage of %u tests/sec.\n\n", count, elapsed, count/elapsed);
+
+                               ftime(ft);
+                               elapsed=(ft->time)-start;
+                               elapsed_u=(ft->millitm)-start_u;
+                               if (elapsed_u < 0)
+	                         {
+	                          elapsed_u=1000+elapsed_u;
+                                  if (elapsed>0) elapsed--;
+	                         }
+                               diff=(elapsed*1000)+elapsed_u;
+		               printf("\n**********************************************\nCollision found ! => %s\n", test);  
+                               printf("\n\nCollision(s) tested : %u in %u second(s), %u millisec.\nAverage of %u tests/sec.\n\n", count, elapsed, elapsed_u, (unsigned int) ((double) (count/diff))*1000);
 
                                       fflush(stdout);
                                       if (!all)
-				         return(result);
+				         return(1);
 				    }
 				}
                               h=(char *)MIN;
@@ -335,15 +377,159 @@ char *crack(input, resume)
       b=(char *)MIN;
     }
 
-  return(NULL);
+  return(0);
+
+}
+
+
+
+int crack_verbose(input, resume)
+     char *input;
+     char *resume;
+{
+  char conv[]="0123456789abcdef";
+  register char *a,*b,*c,*d,*e,*f,*g,*h;
+  unsigned int *result2;
+  char *copy;
+  register int MAX=(unsigned int) alfa+strlen(alfa+1)+1;
+  register int MIN=(unsigned int) alfa+1; 
+  unsigned char i,j; 
+  char *mov;
+  MD5_CTX context;
+  char digest[16];
+  unsigned int digest2[4];
+  struct timeb *tp;
+  struct timeb *ft;
+  unsigned long diff;
+ 
+  ft=(struct timeb *)malloc(sizeof(struct timeb));
+  tp=(struct timeb *)malloc(sizeof(struct timeb)); 
+  bzero(digest, 16);
+  mov=digest;
+  copy=input;
+  strncpy(target, input, 32);
+  
+    a=b=c=d=e=f=g=alfa;
+    if (resume[0])
+    {
+     for (i=0; i<strlen(resume); i++)
+       for (j=0;j<strlen(alfa+1);j++)
+         if (resume[i]==alfa[j])
+            { 
+             if (!i)   h=alfa+j;
+             if (i==1) g=alfa+j;
+             if (i==2) f=alfa+j;
+             if (i==3) e=alfa+j;
+             if (i==4) d=alfa+j;
+             if (i==5) c=alfa+j;
+             if (i==6) b=alfa+j;
+             if (i==7) a=alfa+j;
+            }
+    }
+  else
+    h=(char *)MIN;
+
+
+  up2low(input);
+  ascii2bin(copy, conv, mov);
+  Decode(digest2, mov, 16);  
+  
+
+  bzero(test, 9);
+  signal(SIGINT, handle);
+  ftime(tp);
+  start=tp->time;
+  start_u=tp->millitm;
+
+  
+
+  for(;a<(char *)MAX;a++)
+    {
+       *(test+7)=*a;
+      for(;b<(char *)MAX;b++)
+	{
+	  *(test+6)=*b;
+	  for(;c<(char *)MAX;c++)
+	    {
+	      *(test+5)=*c;
+	      for(;d<(char *)MAX;d++)
+		{
+		  *(test+4)=*d;
+		  for(;e<(char *)MAX;e++)
+		    {
+		      *(test+3)=*e;
+		      for(;f<(char *)MAX;f++)
+			{
+			  *(test+2)=*f;
+			  for(;g<(char *)MAX;g++)
+			    {
+			      *(test+1)=*g;
+			      for(;h<(char *)MAX;h++)
+				{
+				  *test=*h;
+				  *(context.count)=*(context.count+1)=0;
+				  *(context.state)=0x67452301;
+				  *(context.state+1)=0xefcdab89;
+				  *(context.state+2)=0x98badcfe;
+				  *(context.state+3)=0x10325476;
+                                  MD5Update (&context, test, strlen(test));
+                                  result2=MD5Final(&context);
+				  puts("\n");
+                                  printf("%s ",test);
+                                  puts(" <-- string.");
+                                  hashprint(result2);
+                                  puts(" <-- hash.");  
+                                  hashprint(digest2);
+                                  puts(" <-- reference.\n\n");
+                                  fflush(stdout);
+                                  count++;
+
+				  if ((*digest2==*result2)&&(*(digest2+1)==*(result2+1))&&(*(digest2+2)==*(result2+2))&&(*(digest2+3)==*(result2+3)))
+				    {
+
+                               ftime(ft);
+                               elapsed=(ft->time)-start;
+                               elapsed_u=(ft->millitm)-start_u;
+                               if (elapsed_u < 0)
+	                         {
+	                          elapsed_u=1000+elapsed_u;
+                                  if (elapsed>0) elapsed--;
+	                         }
+                               diff=(elapsed*1000)+elapsed_u;
+		               printf("\n**********************************************\nCollision found ! => %s\n", test);  
+                               printf("\n\nCollision(s) tested : %u in %u second(s), %u millisec.\nAverage of %u tests/sec.\n\n", count, elapsed, elapsed_u, (unsigned int) ((double) (count/diff))*1000);
+
+                                      fflush(stdout);
+                                      if (!all)
+				         return(1);
+				    }
+				}
+                              h=(char *)MIN;
+			    }
+                           g=(char *)MIN;
+			}
+                       f=(char *)MIN;
+		    }
+                   e=(char *)MIN;
+		}
+              d=(char *)MIN;
+	    }
+          c=(char *)MIN;
+	}
+      b=(char *)MIN;
+    }
+
+  return(0);
 
 }
 
 void hashprint (digest)
-unsigned char digest[16];
+unsigned int digest[4];
 {
   int i;
+  char conv[16];
 
+  Encode(conv, digest, 16);
   for (i = 0; i < 16; i++)
-    printf ("%02x", digest[i]);
+    printf ("%x", conv[i]);
 }
